﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Internal;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using OrBit.MESInterface.Domain.File.Entities;
using OrBit.MESInterface.Domain.File.Service;
using OrBit.MESInterface.Infrastructure;
using OrBit.MESInterface.Infrastructure.Common;
using OrBit.MESInterface.Infrastructure.Extensions;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Security.Cryptography;
using System.Threading.Tasks;
using OrBit.MESInterface.Application.Contracts.ApiFile;
using OrBit.MESInterface.Application.Contracts.ApiFile.Dtos;
using Volo.Abp.Application.Services;
using Volo.Abp.Threading;

namespace OrBit.MESInterface.Application.Service.ApiFile
{
    public class APIFileAppService : ApplicationService, IAPIFileAppService
    {
        private readonly APIFileManager _apiFileManager;
        private readonly ILogger<APIFileAppService> logger;

        private readonly DefaultHttpClientHelper _httpClient;

        private UploadFileConfig uploadFileConfig;

        public APIFileAppService(APIFileManager fileManager, ILogger<APIFileAppService> logger, DefaultHttpClientHelper _httpClient,
           IOptions<UploadFileConfig> options)
        {
            _apiFileManager = fileManager;
            this.logger = logger;
            this._httpClient = _httpClient;
            uploadFileConfig = options.Value;
        }

        public async Task<ExecutionResult> DownloadAndSaveFileInfo(string url, string fileName, string userName)
        {
            if (string.IsNullOrWhiteSpace(url))
                return ExecutionResult.Failed("下载链接不可为空");
            var downloadInfo = await _httpClient.client.GetAsync(url);

            if (!downloadInfo.IsSuccessStatusCode)
            {
                return ExecutionResult.Failed($"文件下载失败{downloadInfo.StatusCode}");
            }
            using var stream = downloadInfo.Content.ReadAsStream();
            //var formFile = new FormFile(stream, 0L, stream.Length, "file", fileName)
            //{
            //    Headers = new HeaderDictionary(),
            //    ContentType = "application/octet-stream"
            //};

            return await UploadAndSaveFileInfo(new UplaodFileInfoDto
            {
                app = "MESInterface",
                contentType = "application/octet-stream",//formFile.ContentType,
                ProxyDownloadFileList = new List<ProxyDownloadFileInfo> { new ProxyDownloadFileInfo { FileName = fileName, FileStream = stream } },
                UploadFolder = "Default",
                UserName = userName
            });
        }

        private void BulidFormDataContent(MultipartFormDataContent content, IFormFileCollection formFiles, List<FileInfoDto> tmpfileInfo)
        {
            formFiles.ToList().ForEach(file =>
            {
                using (MemoryStream ms = new MemoryStream())
                {
                    file.CopyTo(ms);
                    var filebt = ms.ToArray();

                    var hashid = Convert.ToHexString(SHA256.HashData(ms.ToArray()));

                    var dbfile = AsyncHelper.RunSync(() => { return _apiFileManager.QueryFileInfoByHASH(hashid, Path.GetExtension(file.FileName)); });

                    var tmpinfo = new FileInfoDto
                    {
                        FileName = file.FileName,//Encoding.UTF8.GetString(file.FileName,),
                        ExtensionName = Path.GetExtension(file.FileName),
                        ContentType = file.ContentType,
                        HashID = hashid,
                        Length = file.Length,
                        FileUrl = "",
                        FileID = "",
                        StoragePath = "",
                    };

                    if (dbfile != null)
                    {
                        tmpinfo.FileID = dbfile.APIFileID;
                        tmpinfo.FileUrl = dbfile.FileUrl;
                        tmpinfo.StoragePath = dbfile.StoragePath;
                        tmpfileInfo.Add(tmpinfo);
                    }
                    else
                        content.Add(new ByteArrayContent(filebt), hashid, file.FileName);
                    // uploadFileRequest.FileList.Add(new ByteArrayContent(filebt), hashid, file.FileName);
                }
            });
        }

        private void BulidFormDataContent(MultipartFormDataContent content, List<ProxyDownloadFileInfo> formFiles, List<FileInfoDto> tmpfileInfo)
        {
            formFiles.ToList().ForEach(file =>
            {
                using (MemoryStream ms = new MemoryStream())
                {
                    file.FileStream.CopyTo(ms);
                    var filebt = ms.ToArray();

                    var hashid = Convert.ToHexString(SHA256.HashData(ms.ToArray()));

                    var dbfile = AsyncHelper.RunSync(() => { return _apiFileManager.QueryFileInfoByHASH(hashid, Path.GetExtension(file.FileName)); });

                    var tmpinfo = new FileInfoDto
                    {
                        FileName = file.FileName,//Encoding.UTF8.GetString(file.FileName,),
                        ExtensionName = Path.GetExtension(file.FileName),
                        ContentType = "application/octet-stream",
                        HashID = hashid,
                        Length = ms.Length,
                        FileUrl = "",
                        FileID = "",
                        StoragePath = "",
                    };

                    if (dbfile != null)
                    {
                        tmpinfo.FileID = dbfile.APIFileID;
                        tmpinfo.FileUrl = dbfile.FileUrl;
                        tmpinfo.StoragePath = dbfile.StoragePath;
                        tmpfileInfo.Add(tmpinfo);
                    }
                    else
                        content.Add(new ByteArrayContent(filebt), hashid, file.FileName);
                    // uploadFileRequest.FileList.Add(new ByteArrayContent(filebt), hashid, file.FileName);
                }
            });
        }

        public async Task<ExecutionResult> UploadAndSaveFileInfo(UplaodFileInfoDto uplaodFileInfo)
        {
            if ((uplaodFileInfo.files == null || uplaodFileInfo.files.Count <= 0) && uplaodFileInfo.ProxyDownloadFileList.Count <= 0)
            {
                logger.LogError("未上传文件文件");

                return ExecutionResult.Failed("没有要上传的文件");
            }
            //鉴权

            //判断上传文件不可超过配置大小
            var fileInfo = "";
            var ExceedLimit = false;

            //获取文件服务节点
            var fileServer = GetDownstream(uploadFileConfig.FileServers);
            if (fileServer == null || string.IsNullOrEmpty(fileServer.Url))
                return ExecutionResult.Failed("文件服务节点未配置");

            var UploadUser = await _apiFileManager.GetUserIDByUserName(uplaodFileInfo.UserName);

            if (string.IsNullOrEmpty(UploadUser))
            {
                return ExecutionResult.Failed($"当前用户[{uplaodFileInfo.UserName}]没有上传文件权限");
            }

            //构建上传文件数据
            UploadFileRequestData uploadFileRequest = new UploadFileRequestData
            {
                App = uplaodFileInfo.app,
                // Files = uplaodFileInfo.files,
                UploadFolder = uplaodFileInfo.UploadFolder,

                Url = $"{fileServer.Url}/uploadfile"
            };
            if (uplaodFileInfo.files != null && uplaodFileInfo.files.Count > 0)
            {
                uplaodFileInfo.files.ToList().ForEach(file =>
                {
                    if (file.Length > uploadFileConfig.LimitSize)
                    {
                        ExceedLimit = true;
                        fileInfo += $"文件:{file.FileName}过大，最大可上传长度{uploadFileConfig.LimitSize / 1024 / 1024}mb\n";
                    }
                });
            }

            if (uplaodFileInfo.ProxyDownloadFileList.Count > 0)
            {
                uplaodFileInfo.ProxyDownloadFileList.ForEach(file =>
                {
                    if (file.FileStream.Length > uploadFileConfig.LimitSize)
                    {
                        ExceedLimit = true;
                        fileInfo += $"文件:{file.FileName}过大，最大可上传长度{uploadFileConfig.LimitSize / 1024 / 1024}mb\n";
                    }
                });
            }

            if (ExceedLimit)
                return ExecutionResult.Failed(fileInfo);
            var tmpfileInfo = new List<FileInfoDto>();
            if (uplaodFileInfo.files != null && uplaodFileInfo.files.Count > 0)
                BulidFormDataContent(uploadFileRequest.FileList, uplaodFileInfo.files, tmpfileInfo);
            if (uplaodFileInfo.ProxyDownloadFileList.Count > 0)
                BulidFormDataContent(uploadFileRequest.FileList, uplaodFileInfo.ProxyDownloadFileList, tmpfileInfo);
            //uplaodFileInfo.files.ToList().ForEach(file =>
            //{
            //    using (MemoryStream ms = new MemoryStream())
            //    {
            //        file.CopyTo(ms);
            //        var filebt = ms.ToArray();

            //        var hashid = Convert.ToHexString(SHA256.HashData(ms.ToArray()));

            //        var dbfile = _apiFileManager.QueryFileInfoByHASH(hashid).Result;

            //        var tmpinfo = new FileInfoDto
            //        {
            //            FileName = file.FileName,//Encoding.UTF8.GetString(file.FileName,),
            //            ExtensionName = Path.GetExtension(file.FileName),
            //            ContentType = file.ContentType,
            //            HashID = hashid,
            //            Length = file.Length,
            //            FileUrl = "",
            //            FileID = "",
            //            StoragePath = "",
            //        };

            //        if (dbfile != null)
            //        {
            //            tmpinfo.FileID = dbfile.APIFileID;
            //            tmpinfo.FileUrl = dbfile.FileUrl;
            //            tmpinfo.StoragePath = dbfile.StoragePath;
            //            tmpfileInfo.Add(tmpinfo);
            //        }
            //        else
            //            uploadFileRequest.FileList.Add(new ByteArrayContent(filebt), hashid, file.FileName);
            //    }
            //});
            //上传文件
            if (uploadFileRequest.FileList == null || uploadFileRequest.FileList.Count() > 0)
            {
                var uploadResult = await _httpClient.Upload(uploadFileRequest);

                if (uploadResult.code == ExecutionState.FAIL)
                    return uploadResult;

                var dbfileInfo = new List<APIFile>();
                var newFiles = uploadResult.data.ConvertToDesiredType<FileInfoDto>();

                newFiles.ForEach(file =>
                {
                    var pkid = _apiFileManager.GetPKID().Result;

                    if (string.IsNullOrEmpty(pkid))
                        pkid = Guid.NewGuid().ToString().Substring(0, 12);

                    var apiFile = new APIFile
                    {
                        APIFileID = pkid,
                        FileName = file.FileName,
                        ContentType = file.ContentType,
                        CreateTime = DateTime.Now,
                        ExtensionName = file.ExtensionName,
                        FileUrl = file.FileUrl,
                        FileDescription = file.FileName,
                        StoragePath = file.StoragePath,
                        StorageServerName = fileServer.ServerName,
                        Length = file.Length,
                        UploadUser = UploadUser,
                        HashID = file.HashID
                    };
                    file.FileID = pkid;

                    dbfileInfo.Add(apiFile);
                });
                _apiFileManager._repository.InsertManyAsync(dbfileInfo, true).Wait();

                tmpfileInfo.AddRange(newFiles);
            }

            return ExecutionResult.Success("文件上传成功", tmpfileInfo);
        }

        /// <summary>
        /// 获取文件服务器节点信息
        /// </summary>
        /// <param name="fileServers"></param>
        /// <returns></returns>
        private FileServers GetDownstream(List<FileServers> fileServers)
        {
            var fileServersIndex = 0;
            if (fileServers.Count <= 1)
            {
                return fileServers[0];
            }
            var localIp = NetworkHelp.GetLocalIp();
            if (localIp.StartsWith("192.168") || localIp.StartsWith("127.0.0"))
                fileServersIndex = 1;
            return fileServers[fileServersIndex];
        }
    }
}