﻿using Microsoft.Extensions.Logging;
using OrBit.MESInterface.Application.Contracts.WGTask;
using OrBit.MESInterface.Application.Contracts.WGTask.Dtos;
using OrBit.MESInterface.Domain.MaterialGroup.Entities;
using OrBit.MESInterface.Domain.MaterialGroup.Service;
using OrBit.MESInterface.Infrastructure;
using OrBit.MESInterface.Infrastructure.Common;
using OrBit.MESInterface.Infrastructure.SharedFolderReadWrite;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.NetworkInformation;
using System.Threading.Tasks;
using Volo.Abp.Application.Services;
using Volo.Abp.Threading;

namespace OrBit.MESInterface.Application.Service.WGTask
{
    public class SendWGTaskService : ApplicationService, ISendWGTaskService
    {
        private readonly MaterialGroupManaager materialGroupManaager;
        private readonly SharedFolderReadWriteService sharedFolderReadWriteService;
        private readonly IGenerateTubeSetDataService generateTubeSetDataService;
        private readonly ILogger<SendWGTaskService> logger;
        private readonly DefaultHttpClientHelper _httpClient;

        public SendWGTaskService(MaterialGroupManaager materialGroupManaager,
            IGenerateTubeSetDataService generateTubeSetDataService,
            ILogger<SendWGTaskService> logger, DefaultHttpClientHelper _httpClient) {
            this.materialGroupManaager = materialGroupManaager;
            this.sharedFolderReadWriteService = new SharedFolderReadWriteService();
            this.generateTubeSetDataService = generateTubeSetDataService;
            this.logger = logger;
            this._httpClient = _httpClient;
        }
        public async Task<ExecutionResult> Send(string assetNumber, string productName, string materialGroupID, string timeBlowOut, bool distributePrograms = true)
        {

            var tubeSetDataInfoDto = await GetTubeInfo(materialGroupID, timeBlowOut);

            var programlist = await materialGroupManaager.QueryProgramListByProductNameAndAssetNumber(productName, assetNumber);

            if (distributePrograms && programlist.Count() <= 0)
            {
                return ExecutionResult.Failed($"当前料号：{productName}在产线：{assetNumber}未查询到需要下发的程序。如果下发程序不是必须的请指定不需要下发程序");
            }



            if (tubeSetDataInfoDto == null || tubeSetDataInfoDto.Tubes == null || tubeSetDataInfoDto.Tubes.Count <= 0)
            {
                logger.LogInformation($"当前组合关系ID{materialGroupID}未查询到详细信息");
                return ExecutionResult.Failed("没有查询到组合关系");
            }

            var filePathInfo = await materialGroupManaager.QueryEqAssetInfo(assetNumber);

            if (filePathInfo == null)
                return ExecutionResult.Failed("设备未维护任务存放路径");

            var sendTaskResult = DistributeTask(tubeSetDataInfoDto,  $"{productName}-{materialGroupID}", filePathInfo.ServerSendTaskPath);
            if (sendTaskResult.code != ExecutionState.SUCCESS)
            {
                DeleteTaskFile(materialGroupID, filePathInfo.ServerSendTaskPath);
                return sendTaskResult;
            }
               
            var sendProgramResult = DistributePrograms(programlist.ToList(), filePathInfo.ServerSendProgramPath);
            if (sendProgramResult.Count>0 && !sendProgramResult.Any(r=>r.code== ExecutionState.OK))
            {
                DeleteTaskFile(materialGroupID, filePathInfo.ServerSendTaskPath);
                DeletePrograms(programlist.ToList(), filePathInfo.ServerSendProgramPath);
                return ExecutionResult.Failed("任务下发成功，但是程序下发失败",sendProgramResult);
            }


            return ExecutionResult.Success("下发成功");
        }

        private ExecutionResult DistributeTask(TubeSetDataInfoDto tubeSetDataInfoDto,string taskFileName,string serverSendTaskPath)
        {
            try
            {
                if(File.Exists(Path.Combine($"{taskFileName}.tac")))
                    File.Delete(Path.Combine( $"{taskFileName}.tac"));
                generateTubeSetDataService.Build(tubeSetDataInfoDto).Save($"{taskFileName}.tac");

                logger.LogInformation($"任务文件存放地址：{serverSendTaskPath}");

                if (!Directory.Exists(serverSendTaskPath))
                {
                    logger.LogInformation($"任务文件存放地址：{serverSendTaskPath}不存在。即将通过命令行打开网络地址");
                    try
                    {
                        sharedFolderReadWriteService.connectState(serverSendTaskPath,"tasksenduser","123456");
                    }
                    catch (Exception ex) {
                        logger.LogError($"下发任务时命令行打开网络路径失败。失败原因{ex.Message}");
                        return ExecutionResult.Failed($"下发任务时命令行打开网络路径失败。失败原因{ex.Message}");
                    }
                }
                DirectoryInfo theFolder = new(serverSendTaskPath);
                //相对共享文件夹的路径
                string fielpath = @"\";
                //获取保存文件的路径
                string filename = theFolder.ToString() + fielpath;
                logger.LogInformation($"准备下发任务文件。文件下发路径：{filename}。文件名：{taskFileName}.tac");

                sharedFolderReadWriteService.Transport($"{taskFileName}.tac", filename, $"{taskFileName}.tac");
            }
            catch (System.Exception ex)
            {
                logger.LogError($"下发任务文件失败。失败原因{ex.Message}");
                return ExecutionResult.Failed(ex.Message);
            }
            finally
            {
                if (File.Exists(Path.Combine($"{taskFileName}.tac")))
                    File.Delete(Path.Combine($"{taskFileName}.tac"));
            }
            logger.LogError($"下发任务文件成功。");
            return ExecutionResult.Success("下发成功");
        }

        private void DeleteTaskFile (string materialGroupID, string serverSendTaskPath)
        {
            try
            {
               var taskFilePath= Path.Combine(serverSendTaskPath, $"{materialGroupID}.tac");
                if(File.Exists(taskFilePath))
                    File.Delete(taskFilePath);
            }
            catch { }
        }
        private void DeletePrograms(List<ProcessStepProgramInfo> programs, string serverSendProgramPath)
        {
            programs.ForEach(program =>
            {
                var description = program.Description;
                if (program.Description.StartsWith("Product_ProgramR"))
                {

                    description = "RoBot" + description.Replace("Product_ProgramR", "") + "\\" + program.Description;
                }
                var savePath = Path.Combine(serverSendProgramPath, description, program.FileName);
                if (File.Exists(savePath))
                {
                    File.Delete(savePath);
                }
            });
        }
        private List<ExecutionResult> DistributePrograms(List<ProcessStepProgramInfo> programs, string serverSendProgramPath)
        {

            if (programs.Count() > 0)
            {
                var listResult = new List<ExecutionResult>();
                if (!Directory.Exists(serverSendProgramPath))
                {
                    try
                    {
                        sharedFolderReadWriteService.connectState(serverSendProgramPath);
                    }
                    catch (Exception ex)
                    {
                        logger.LogError($"下发程序时命令行打开网络路径失败。失败原因{ex.Message}");
                    }
                }
                for (int i = 0; i < programs.Count; i++)
                {

                    var program = programs[i];
                    var description = program.Description;
                    if (program.Description.StartsWith("Product_ProgramR"))
                    {

                        description = "RoBot" + description.Replace("Product_ProgramR", "") + "\\" + program.Description;
                    }
                    var savePath = Path.Combine(serverSendProgramPath, description);

                    if (!Directory.Exists(savePath))
                    {
                        Directory.CreateDirectory(savePath);
                    }
                    listResult.Add(AsyncHelper.RunSync(() => { return _httpClient.DownLoad(program.FileUrl, Path.Combine(savePath, program.FileName)); }));

                    break;
                }
              
                return listResult;
            }
            else
                return new List<ExecutionResult>();


        }


        private async Task<TubeSetDataInfoDto> GetTubeInfo(string materialGroupID,string timeBlowOut)
        {
            var materia = await materialGroupManaager.QueryMaterialGroupById(materialGroupID);

            if (materia == null || materia.MaterialGroupInfoTaskListInfos.Count() <= 0)
                return null ;

            List<TubeInfoDto> tubeInfoDtos = new List<TubeInfoDto>();
            materia.MaterialGroupInfoTaskListInfos.ToList().ForEach(info =>
            {
                tubeInfoDtos.Add(new TubeInfoDto
                {
                    no = (tubeInfoDtos.Count() + 1).ToString(),
                    Length = info.Length.ToString(),
                    Pieces = info.Count.ToString(),
                    ProductName = info.ProductName.Replace('.','_'),
                    Storage = info.Storage,
                    Support = info.Support
                });
            });
            TubeSetDataInfoDto tubeSetDataInfoDto = new TubeSetDataInfoDto
            {
                Waste = true,
                Name = materialGroupID,
                LengthInLoading = materia.Length,
                TimeBlowOut = timeBlowOut,
                NumberOfTubeSets = materia.count.ToString()
            };
            tubeSetDataInfoDto.Tubes = tubeInfoDtos;

            return tubeSetDataInfoDto;
        }

    }
}
