﻿using Microsoft.Data.SqlClient;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Data;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using OrBit.MESInterface.Infrastructure.Extensions;
using Volo.Abp.EntityFrameworkCore;

namespace OrBit.MESInterface.EntityFrameworkCore.Extensions
{
    public static class EFContextExtends
    {
      
        /// <summary>
        /// 执行sql返回DataSet
        /// </summary>
        /// <param name="dbContext"></param>
        /// <param name="sql"></param>
        /// <param name="commandParameter"></param>
        /// <returns></returns>
        public static DataSet ExecuteSqlResultDataSet(this IEfCoreDbContext dbContext, string sql, SqlParameter[] commandParameter)
        {
            return GetDataSet(dbContext.Database.GetDbConnection().ConnectionString, CommandType.Text, sql, commandParameter);
        }
        public static  Task<DataTable> ExecuteSqlResultDataTable(this IEfCoreDbContext dbContext, string sql, params SqlParameter[] parameters) 
        {
            return GetTable(dbContext, CommandType.Text, sql, parameters);
          
        }
        /// <summary>
        /// 执行sql 返回集合
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="dbContext"></param>
        /// <param name="sql"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<IEnumerable<T>> ExecuteSqlResultEnumerable<T>(this IEfCoreDbContext dbContext, string sql, params SqlParameter[] parameters) where T : class, new()
        {
            DataTable dt = await GetTable(dbContext, CommandType.Text,sql, parameters);
            return dt.ToEnumerable<T>();
        }


        /// <summary>
        /// 存储过程专用
        /// </summary>
        /// <param name="cmdText">存储过程的名字或者 T-SQL 语句</param>
        /// <param name="commandParameters">以数组形式提供SqlCommand命令中用到的参数列表</param>
        /// <returns>返回一个表集合(DataTableCollection)表示查询得到的数据集</returns>
        public static DataSet ExecuteStoredProcedureResultDataSet(this IEfCoreDbContext dbContext, string sql, SqlParameter[] commandParameter = default)
        {
            return GetDataSet(dbContext.Database.GetDbConnection().ConnectionString, CommandType.StoredProcedure, sql, commandParameter);
        }


        /// <summary>
        /// 执行数据库命令获取DataTable
        /// </summary>
        /// <param name="dbContext"></param>
        /// <param name="cmdTye"></param>
        /// <param name="cmdText"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<DataTable> GetTable(this IEfCoreDbContext dbContext, CommandType cmdTye, string cmdText, params SqlParameter[] parameters)
        {
            SqlCommand cmd = new SqlCommand();
            DataTable dt = new DataTable();
            using (SqlConnection conn = new SqlConnection(dbContext.Database.GetDbConnection().ConnectionString))
            {
                PrepareCommand(cmd, conn, null, cmdTye, cmdText, parameters);
                DbDataReader reader = await cmd.ExecuteReaderAsync();
                dt.Load(reader);
                reader.Close();
                cmd.Parameters.Clear();
            }
            return dt;
        }

       

        /// <summary>
        /// 执行数据库命令获取DataSet
        /// </summary>
        /// <param name="connecttionString"></param>
        /// <param name="cmdTye"></param>
        /// <param name="cmdText"></param>
        /// <param name="commandParameters"></param>
        /// <returns></returns>
        private static DataSet GetDataSet(string connecttionString, CommandType cmdTye, string cmdText, SqlParameter[] commandParameters = default)
        {
            SqlCommand cmd = new SqlCommand();
            DataSet ds = new DataSet();
            using (SqlConnection conn = new SqlConnection(connecttionString))
            {
                PrepareCommand(cmd, conn, null, cmdTye, cmdText, commandParameters);
                SqlDataAdapter adapter = new SqlDataAdapter();
                adapter.SelectCommand = cmd;
                adapter.Fill(ds);

                cmd.Parameters.Clear();

            }
            return ds;
        }

        /// <summary>
        /// 执行非查询操作
        /// </summary>
        /// <param name="cmdText">sql语句</param>
        /// <param name="param">Sql语句参数</param>
        /// <returns></returns>
        public static int NonQuery(this IEfCoreDbContext dbContext, CommandType cmdTye, string cmdText, SqlParameter[] param = null)
        {
            SqlConnection conn = new SqlConnection(dbContext.Database.GetDbConnection().ConnectionString);
            SqlCommand cmd = new SqlCommand();            
            try
            {
                PrepareCommand(cmd, conn, null, cmdTye, cmdText, param);
                conn.Open();
                return cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw new Exception("ExecuteNonQuery()异常：" + ex.Message);
            }
            finally   //以上不管是否发生异常，都会执行的代码
            {
                conn.Close();
            }
        }


        private static void PrepareCommand(SqlCommand cmd, SqlConnection conn, SqlTransaction trans, CommandType cmdType, string cmdText, SqlParameter[] cmdParms = default)
        {
            //判断数据库连接状态
            if (conn.State != ConnectionState.Open)
                conn.Open();
            cmd.Connection = conn;
            cmd.CommandTimeout = 60000;
            cmd.CommandText = cmdText;
            //判断是否需要事物处理
            if (trans != null)
                cmd.Transaction = trans;
            cmd.CommandType = cmdType;
            if(cmdParms!=null)
            cmd.Parameters.AddRange(cmdParms);
            
        }
    }
}
