﻿using Azure.Core;
using Microsoft.AspNetCore.Http;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Net.Mail;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Reflection.PortableExecutable;
using System.Text;
using System.Threading.Tasks;

namespace OrBit.MESInterface.Infrastructure.Common
{
    public class DefaultHttpClientHelper
    {
        public HttpClient client { get; private set; }

        public DefaultHttpClientHelper(HttpClient httpClient)
        {
            client = httpClient;
        }

        public async Task<ExecutionResult> Upload(UploadFileRequestData uploadFile)
        {
            if (uploadFile == null)
            {
                throw new ArgumentNullException(nameof(uploadFile));
            }
            if (uploadFile.FileList.Count() <= 0)
                return ExecutionResult.Failed("没有文件需要上传");
            var content = uploadFile.FileList;

            content.Headers.Add("app", uploadFile.App);
            content.Headers.Add("UploadFolder", uploadFile.UploadFolder);

            var localIp = NetworkHelp.GetLocalIp();

            content.Headers.Add("X-Forwarded-For", localIp);
            var response = await client.PostAsync(uploadFile.Url, content);

            if (!response.IsSuccessStatusCode)
            {
                return ExecutionResult.Failed($"上传失败,失败原因:{response.StatusCode}");
            }

            var result = await response.Content.ReadAsStringAsync();

            return JsonConvert.DeserializeObject<ExecutionResult>(result);
        }

        public async Task<ExecutionResult> Request(RequestData requestData)
        {
            HttpContent httpContent = new StringContent(requestData.PostData, Encoding.UTF8);

            HttpResponseMessage response = null;
            var analyticresult = new ExecutionResult();

            if (requestData.IsBasicAuth)
            {
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", Convert.ToBase64String(Encoding.UTF8.GetBytes($"{requestData.BasicAuthUser}:{requestData.BasicAuthPassword}")));
            }
            if (requestData.Headers.ContainsKey("Authorization"))
            {
                client.DefaultRequestHeaders.Authorization = new System.Net.Http.Headers.AuthenticationHeaderValue("Bearer", requestData.Headers["Authorization"]);
            }
            FormatRequestHeaders(requestData.Headers, httpContent.Headers);

            if (requestData.RequestMethod == RequestMethodType.POST)
                response = await client.PostAsync(requestData.Url, httpContent);
            else
            {
                string url = "";
                if (requestData.Parameters != null && requestData.Parameters.Count > 0)
                {
                    url = FormatGetParametersToUrl(requestData.Url, requestData.Parameters, requestData.dataEncoding);
                }
                response = await client.GetAsync(url);
            }
            return ExecutionResult.Success(response.StatusCode.ToString(), await response.Content.ReadAsStringAsync());

        }

        private string FormatGetParametersToUrl(string url, IDictionary<string, string> parameters, Encoding paraEncoding)
        {
            if (url.IndexOf("?") < 0)
                url += "?";
            int i = 0;
            string sendContext = "";
            foreach (var parameter in parameters)
            {
                if (i > 0)
                {
                    sendContext += "&";
                }

                sendContext += parameter.Key// HttpUtility.UrlEncode(parameter.Key, paraEncoding)
                               + "=" + parameter.Value; //HttpUtility.UrlEncode(parameter.Value, paraEncoding);
                ++i;
            }

            url += sendContext;
            return url;
        }

        /// <summary>
        /// 格式化请求头信息
        /// </summary>
        /// <param name="headers"></param>
        /// <param name="request"></param>
        private void FormatRequestHeaders(IDictionary<string, string> headers, HttpContentHeaders Header)
        {
            if (headers != null && headers.Count() > 0)
            {
                foreach (var hd in headers)
                {
                    if (hd.Key.Contains("Authorization"))
                        continue;
                    if (Header.Contains(hd.Key))
                        Header.Remove(hd.Key);
                    Header.TryAddWithoutValidation(hd.Key, hd.Value);
                }
            }
        }


        /// <summary>
        /// 下载文件
        /// </summary>
        /// <param name="serverFileName">服务器上文件名 如 close.png</param>
        /// <param name="localFileName">要保存到本地的路径全名 如：C://Download/close.png</param>
        /// <returns></returns>
        public async Task<ExecutionResult> DownLoad(string uri, string localFileName)
        {
            var server = new Uri(uri);
            var p = Path.GetDirectoryName(localFileName);
            if (!Directory.Exists(p)) Directory.CreateDirectory(p);

            // 发起请求并异步等待结果
            
            var responseMessage =await client.GetAsync(server);
            if (responseMessage.IsSuccessStatusCode)
            {
                if(File.Exists(localFileName))
                    File.Delete(localFileName);
                using (var fs = File.Create(localFileName))
                {
                    // 获取结果，并转成 stream 保存到本地。
                    var streamFromService =await responseMessage.Content.ReadAsStreamAsync();
                    streamFromService.CopyTo(fs);
                    return ExecutionResult.Success($"文件下载成功，保存为：{localFileName}");
                }
            }
            else
                return ExecutionResult.Failed($"文件下载失败 {responseMessage.StatusCode}");
        }


    }
}