﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace OrBit.MESInterface.Infrastructure.Common
{
    public static class HttpRequestHelper
    {
        /// <summary>
        /// http/https请求响应
        /// </summary>
        /// <param name="getOrPost"></param>
        /// <param name="url">地址（要带上http或https）</param>
        /// <param name="headers">请求头</param>
        /// <param name="parameters">提交数据</param>
        /// <param name="dataEncoding">编码类型 utf-8</param>
        /// <param name="contentType">application/x-www-form-urlencoded</param>
        /// <returns></returns>
        public static HttpWebResponse HttpRequest(RequestData requestData)
        {
            var request = CreateRequest(requestData.RequestMethod, requestData.Url, requestData.Headers, requestData.Parameters, requestData.dataEncoding, requestData.ContentType);

            //如果需要POST数据
            if (requestData.RequestMethod == RequestMethodType.POST)
            {
                var data = FormatPostParameters(requestData.PostData, requestData.dataEncoding);
                if (requestData.IsBasicAuth)
                {
                    string usernamePassword = requestData.BasicAuthUser + ":" + requestData.BasicAuthUser;
                    CredentialCache credentialCache =
                        new CredentialCache { { new Uri(requestData.Url), "Basic", new NetworkCredential(requestData.BasicAuthUser, requestData.BasicAuthPassword) } };
                    request.Credentials = credentialCache;
                    request.Headers.Add("Authorization",
                        "Basic " + Convert.ToBase64String(new ASCIIEncoding().GetBytes(usernamePassword)));
                }

                using (Stream stream = request.GetRequestStream())
                {
                    stream.Write(data, 0, data.Length);
                    stream.Close();
                }
            }

            WebResponse Res = null;
            try
            {
                Res = request.GetResponse();
            }
            catch (WebException ex)
            {
                Res = (HttpWebResponse)ex.Response;
            }
            catch (Exception e)
            {
                throw e;
            }

            if (null == Res)
            {
                return request.GetResponse() as HttpWebResponse;
            }

            return (HttpWebResponse)Res;
        }

        public static string AnalysisResponse(HttpWebResponse response)
        {
            var respHTML = string.Empty;
            try
            {
                if (response.StatusCode == HttpStatusCode.OK)
                {
                    System.IO.StreamReader readerGET;
                    readerGET = new System.IO.StreamReader(response.GetResponseStream(), Encoding.UTF8);
                    respHTML = readerGET.ReadToEnd();  //得到响应结果
                    readerGET.Close();
                    response.Close();
                }
                else
                {
                    respHTML = response.StatusDescription;
                }
            }
            catch (Exception ex)
            {
                respHTML = ex.Message;
            }

            return respHTML;
        }

        /// <summary>
        /// 创建HTTP请求对象
        /// </summary>
        /// <param name="getOrPost"></param>
        /// <param name="url"></param>
        /// <param name="headers"></param>
        /// <param name="parameters"></param>
        /// <param name="paraEncoding"></param>
        /// <param name="contentType"></param>
        /// <returns></returns>
        private static HttpWebRequest CreateRequest(
            RequestMethodType RequestMethod
            , string url
            , IDictionary<string, string> headers
            , IDictionary<string, string> parameters
            , Encoding paraEncoding
            , string contentType
            )
        {
            if (string.IsNullOrEmpty(url))
            {
                throw new ArgumentNullException("url");
            }

            if (parameters != null && parameters.Count > 0 && paraEncoding == null)
            {
                throw new ArgumentNullException("requestEncoding");
            }

            HttpWebRequest request = null;

            if (parameters != null && parameters.Count > 0)
            {
                url = FormatGetParametersToUrl(url, parameters, paraEncoding);
            }

            request = WebRequest.Create(url) as HttpWebRequest;

            //判断是否是https
            if (url.StartsWith("https", StringComparison.OrdinalIgnoreCase))
            {
                ServicePointManager.ServerCertificateValidationCallback =
                    new RemoteCertificateValidationCallback(CheckValidationResult);

                request.ProtocolVersion = HttpVersion.Version10;
            }
            request.Method = RequestMethod.ToString();

            if (contentType == null)
            {
                request.ContentType = "application/x-www-form-urlencoded;charset=UTF-8";
            }
            else
            {
                request.ContentType = contentType;
            }

            //POST的数据大于1024字节的时候，如果不设置会分两步
            request.ServicePoint.Expect100Continue = false;
            request.ServicePoint.ConnectionLimit = int.MaxValue;

            if (headers != null)
            {
                FormatRequestHeaders(headers, request);
            }
            request.Timeout = 300000;
            return request;
        }

        /// <summary>
        /// 格式化请求头信息
        /// </summary>
        /// <param name="headers"></param>
        /// <param name="request"></param>
        private static void FormatRequestHeaders(IDictionary<string, string> headers, HttpWebRequest request)
        {
            foreach (var hd in headers)
            {
                //因为HttpWebRequest中很多标准标头都被封装成只能通过属性设置，添加的话会抛出异常
                switch (hd.Key.ToLower())
                {
                    case "connection":
                        request.KeepAlive = false;
                        break;

                    case "content-type":
                        request.ContentType = hd.Value;
                        break;

                    case "transfer-enconding":
                        request.TransferEncoding = hd.Value;
                        break;

                    default:
                        request.Headers.Add(hd.Key, hd.Value);
                        break;
                }
            }
        }

        /// <summary>
        /// 格式化Get请求参数
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="parameters">参数</param>
        /// <param name="paraEncoding">编码格式</param>
        /// <returns></returns>

        private static string FormatGetParametersToUrl(string url, IDictionary<string, string> parameters, Encoding paraEncoding)
        {
            if (url.IndexOf("?") < 0)
                url += "?";
            int i = 0;
            string sendContext = "";
            foreach (var parameter in parameters)
            {
                if (i > 0)
                {
                    sendContext += "&";
                }

                sendContext += parameter.Key// HttpUtility.UrlEncode(parameter.Key, paraEncoding)
                       + "=" + parameter.Value; //HttpUtility.UrlEncode(parameter.Value, paraEncoding);
                ++i;
            }

            url += sendContext;
            return url;
        }

        /// <summary>
        /// 格式化Post请求参数
        /// </summary>
        /// <param name="PostData"></param>
        /// <param name="dataEncoding"></param>
        /// <returns></returns>
        private static byte[] FormatPostParameters(string PostData, Encoding dataEncoding)
        {
            byte[] data = dataEncoding.GetBytes(PostData);
            return data;
        }

        private static bool CheckValidationResult(object sender, X509Certificate certificate, X509Chain chain,
            SslPolicyErrors errors)
        {
            return true; //总是接受
        }
    }
}