using System.Data;
using System.Data.Common;
using System.Reflection;
using Microsoft.Data.SqlClient;
using Microsoft.EntityFrameworkCore;

namespace OrBit.MESInterface.OrderVerifyWebService.EntiyFrameworkCore.Extensions;

public static class EFContextExtends
{
    public static Task<DataSet> SqlQueryResultDataSetAsync(this DbContext dbContext, string sql, SqlParameter[] commandParameter = default)
    {
        return Task.Run(() =>
        {
            return GetTable(dbContext.Database.GetDbConnection().ConnectionString, CommandType.Text, sql, commandParameter);
        });
    }

    public static DataSet SqlQueryResultDataSet(this DbContext dbContext, string sql, SqlParameter[] commandParameter)
    {
        return GetTable(dbContext.Database.GetDbConnection().ConnectionString, CommandType.Text, sql, commandParameter);
    }

    public static async Task<IEnumerable<T>> SqlQuery<T>(this DbContext dbContext, string sql, params SqlParameter[] parameters) where T : class, new()
    {
        DataTable dt = await SqlQuery(dbContext, sql, parameters);
        return dt.ToEnumerable<T>();
    }

    public static IEnumerable<T> ToEnumerable<T>(this DataTable dt) where T : class, new()
    {
        PropertyInfo[] propertyInfos = typeof(T).GetProperties();
        T[] ts = new T[dt.Rows.Count];
        int i = 0;
        foreach (DataRow row in dt.Rows)
        {
            T t = new T();
            foreach (PropertyInfo p in propertyInfos)
            {
                if (dt.Columns.IndexOf(p.Name) != -1 && row[p.Name] != DBNull.Value)
                    p.SetValue(t, row[p.Name], null);
            }
            ts[i] = t;
            i++;
        }
        return ts;
    }

    /// <summary>
    /// 存储过程专用
    /// </summary>
    /// <param name="cmdText">存储过程的名字或者 T-SQL 语句</param>
    /// <param name="commandParameters">以数组形式提供SqlCommand命令中用到的参数列表</param>
    /// <returns>返回一个表集合(DataTableCollection)表示查询得到的数据集</returns>
    public static DataSet ExecuteStoredProcedureResultDataSet(this DbContext dbContext, string sql, SqlParameter[] commandParameter)
    {
        return GetTable(dbContext.Database.GetDbConnection().ConnectionString, CommandType.StoredProcedure, sql, commandParameter);
    }

    /// <summary>
    /// 存储过程专用
    /// </summary>
    /// <param name="cmdText">存储过程的名字或者 T-SQL 语句</param>
    /// <param name="commandParameters">以数组形式提供SqlCommand命令中用到的参数列表</param>
    /// <returns>返回一个表集合(DataTableCollection)表示查询得到的数据集</returns>
    public static Task<DataSet> ExecuteStoredProcedureResultDataSetAsync(this DbContext dbContext, string sql, SqlParameter[] commandParameter)
    {
        return Task.Run(() =>
        {
            return GetTable(dbContext.Database.GetDbConnection().ConnectionString, CommandType.StoredProcedure, sql, commandParameter);
        });
    }

    public static async Task<DataTable> SqlQuery(this DbContext dbContext, string sql, params SqlParameter[] parameters)
    {
        SqlCommand cmd = new SqlCommand();
        DataTable dt = new DataTable();
        using (SqlConnection conn = new SqlConnection(dbContext.Database.GetDbConnection().ConnectionString))
        {
            PrepareCommand(cmd, conn, null, CommandType.Text, sql, parameters);
            DbDataReader reader = await cmd.ExecuteReaderAsync();
            dt.Load(reader);
            reader.Close();
            cmd.Parameters.Clear();
        }

        return dt;
    }

    /// <summary>
    /// 存储过程专用
    /// </summary>
    /// <param name="dbContext">dbContext</param>
    /// <param name="sql">存储过程名称</param>
    /// <param name="parameters"></param>
    /// <returns></returns>
    public static async Task<int> ExecuteStoredProcedureNonQuery(this DbContext dbContext, string sql, params SqlParameter[] parameters)
    {
        SqlCommand cmd = new SqlCommand();
        int count = 0;
        using (SqlConnection conn = new SqlConnection(dbContext.Database.GetDbConnection().ConnectionString))
        {
            PrepareCommand(cmd, conn, null, CommandType.StoredProcedure, sql, parameters);
            count = await cmd.ExecuteNonQueryAsync();

            cmd.Parameters.Clear();
        }

        return count;
    }

    private static DataSet GetTable(string connecttionString, CommandType cmdTye, string cmdText, SqlParameter[] commandParameters)
    {
        SqlCommand cmd = new SqlCommand();
        DataSet ds = new DataSet();
        using (SqlConnection conn = new SqlConnection(connecttionString))
        {
            PrepareCommand(cmd, conn, null, cmdTye, cmdText, commandParameters);
            SqlDataAdapter adapter = new SqlDataAdapter();
            adapter.SelectCommand = cmd;
            adapter.Fill(ds);

            cmd.Parameters.Clear();
        }
        return ds;
    }

    private static void PrepareCommand(SqlCommand cmd, SqlConnection conn, SqlTransaction trans, CommandType cmdType, string cmdText, SqlParameter[] cmdParms)
    {
        //判断数据库连接状态
        if (conn.State != ConnectionState.Open)
            conn.Open();
        cmd.Connection = conn;
        cmd.CommandTimeout = 60000;
        cmd.CommandText = cmdText;
        //判断是否需要事物处理
        if (trans != null)
            cmd.Transaction = trans;
        cmd.CommandType = cmdType;
        if (cmdParms != null)
        {
            foreach (SqlParameter parm in cmdParms)
                if (!cmd.Parameters.Contains(parm))
                    cmd.Parameters.Add(parm);
        }
    }
}